//--------------------
// Name: Sassy
// Description: Saving System
// Version: 1.1
// Author: Jookia
//--------------------

// Update 1.1
//  Cleaned up the code style.
//  Added error stacks.
//  Added a pref array feature.

function Sassy::onAdd(%this)
{
	%errorStack = "";
	
	if(%this.getName() $= "")
		%errorStack = "ERROR: %1::onAdd(): This database has no name!";
	
	if($server::lan == 1)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: %1::onAdd(): Sassy does not support LAN servers!";
	
	if(%this.dataFile $= "")
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: %1::onAdd(): Sassy needs a dataFile variable!";
	
	if(%errorStack !$= "")
	{
		echo(strReplace(%errorStack, "%1", (%this.getName() $= "") ? "Sassy" : %this.getName()));
		
		%this.schedule(0, "delete");
		
		return false;
	}
	
	%this.valueCount = 0;
	%this.dataCount = 0;
	
	if(isFile(%this.dataFile) == 1)
	{
		echo(%this.getName() @ "::onAdd(): Previous save file found. Loading..");
		
		%this.loadData();
	}
	
	return true;
}

function Sassy::onRemove(%this)
{
	for(%b = 1; %b <= %this.dataCount; %b++)
		%this.data[%b].delete();
	
	return true;
}

function Sassy::saveData(%this)
{
	%file = new fileObject();
	%file.openForWrite(%this.dataFile);
	
	%file.writeLine("values");
	
	for(%a = 1; %a <= %this.valueCount; %a++)
		%file.writeLine(" " @ %this.value[%a] SPC %this.defaultValue[%a]);
	
	if(%this.dataCount > 0)
		%file.writeLine("");
	
	for(%b = 1; %b <= %this.dataCount; %b++)
	{
		if(isObject(%this.data[%b]) == 0)
			continue;
		
		%file.writeLine("ID " @ %this.data[%b].ID);
		
		for(%c = 1; %c <= %this.valueCount; %c++)
			%file.writeLine(" " @ %this.value[%c] SPC %this.data[%b].value[%this.value[%c]]);
		
		if(%b < %this.dataCount)
			%file.writeLine("");
	}
	
	%file.close();
	%file.delete();
	
	return true;
}

function Sassy::loadData(%this)
{
	if(isFile(%this.dataFile) == 0)
	{
		echo("ERROR: " @ %this.getName() @ "::loadData(): File '" @ %this.dataFile @ "' not found.");
		
		return false;
	}
	
	%file = new fileObject();
	%file.openForRead(%this.dataFile);
	
	%currentState = "";
	
	while(%file.isEOF() == 0)
	{
		%line = %file.readLine();
		
		if(getWord(%line, 0) $= "" && getWord(%line, 1) $= "")
			continue;
		
		if(getSubStr(%line, 0, 1) $= " ")
		{
			%line = getSubStr(%line, 1, strLen(%line) - 1);
			
			if(%currentState $= "values")
			{
				%this.valueCount++;
				
				%this.value[%this.valueCount] = getWord(%line, 0);
				%this.defaultValue[%this.valueCount] = getWords(%line, 1, getWordCount(%line) - 1);
			}
			
			if(%currentState $= "ID")
				%this.data[%this.dataCount].value[getWord(%line, 0)] = getWords(%line, 1, getWordCount(%line) - 1);
			
			continue;
		}
		
		%currentState = getWord(%line, 0);
		
		if(getWord(%line, 0) $= "ID")
		{
			if(%this.getData(getWord(%line, 1)) == -1)
			{
				%this.dataCount++;
				
				%this.data[%this.dataCount] = new scriptObject()
				{
					class = SassyData;
					
					ID = getWord(%line, 1);
					parent = %this;
				};
			}
		}
	}
	
	%file.close();
	%file.delete();
	
	return true;
}

function Sassy::addValue(%this, %value, %defaultValue)
{
	%errorStack = "";
	
	if(%value $= "" || %defaultValue $= "")
		%errorStack = "ERROR: " @ %this.getName() @ "::addValue([value: " @ %value @ "], [defaultValue: " @ %defaultValue @ "]): Incorrect amount of arguments!";
	
	%bail = false;
	
	for(%a = 0; %a <= %this.valueCount; %a++)
		if(%this.value[%a] $= %value)
			%bail = true;
	
	if(%bail == true)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: " @ %this.getName() @ "::addValue([value: " @ %value @ "], [defaultValue: " @ %defaultValue @ "]): Value '" @ %value @ "' is already in the database!";
	
	if(getWordCount(%value) > 1)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: " @ %this.getName() @ "::addValue([value: " @ %value @ "], [defaultValue: " @ %defaultValue @ "]): Values can't be longer then one word!";
	
	if(%errorStack !$= "")
	{
		echo(%errorStack);
		
		return false;
	}
	
	%this.valueCount++;
	
	%this.value[%this.valueCount] = %value;
	%this.defaultValue[%this.valueCount] = %defaultValue;
	
	for(%a = 1; %a <= %this.dataCount; %a++)
		%this.data[%a].value[%value] = %defaultValue;
	
	return true;
}

function Sassy::removeValue(%this, %value)
{
	%errorStack = "";
	
	if(%value $= "")
		%errorStack = "ERROR: " @ %this.getName() @ "::removeValue(): Incorrect amount of arguments!";
	
	%bail = true;
	
	for(%a = 0; %a <= %this.valueCount; %a++)
		if(%this.value[%a] $= %value)
			%bail = false;
	
	if(%bail == true)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: " @ %this.getName() @ "::removeValue(): Value '" @ %value @ "' is not found in the database!";
	
	if(%errorStack !$= "")
	{
		echo(%errorStack);
		
		return false;
	}
	
	%foundValue = false;
	
	for(%a = 0; %a < %this.valueCount; %a++)
	{
		if(%this.value[%a] $= %value)
		{
			%foundValue = true;
			
			%this.value[%a] = "";
			%this.defaultValue[%a] = "";
			
			continue;
		}
		
		if(%foundValue == true)
		{
			%this.value[%a - 1] = %this.value[%a];
			%this.defaultValue[%a - 1] = %this.defaultValue[%a];
			
			%this.value[%a] = "";
			%this.defaultValue[%a] = "";
		}
	}
	
	%this.valueCount--;
	
	for(%b = 1; %b <= %this.dataCount; %b++)
		%this.data[%b].value[%value] = "";
	
	return true;
}

function Sassy::addData(%this, %ID)
{
	%errorStack = "";
	
	if(%ID $= "")
		%errorStack = "ERROR: " @ %this.getName() @ "::addData([ID: " @ %ID @ "]): Incorrect amount of arguments!";
	
	if(%this.getData(%ID) != -1)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: " @ %this.getName() @ "::addData([ID: " @ %ID @ "]): Data for ID '" @ %ID @ "' is already in the database!";
	
	if(%errorStack !$= "")
	{
		echo(%errorStack);
		
		return false;
	}
	
	%data = new scriptObject()
	{
		class = SassyData;
		
		ID = %ID;
		parent = %this;
	};
	
	%this.dataCount++;
	%this.data[%this.dataCount] = %data;
	
	return true;
}

function Sassy::removeData(%this, %ID)
{
	%errorStack = "";
	
	if(%ID $= "")
		%errorStack = "ERROR: " @ %this.getName() @ "::removeData([ID: " @ %ID @ "]): Incorrect amount of arguments!";
	
	if(%this.getData(%ID) == -1)
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: " @ %this.getName() @ "::removeData([ID: " @ %ID @ "]): Data for ID '" @ %ID @ "' is not found in the database!";
	
	if(%errorStack !$= "")
	{
		echo(%errorStack);
		
		return false;
	}
	
	%foundID = false;
	
	for(%a = 1; %a <= %this.dataCount; %a++)
	{
		if(%this.data[%a].ID == %ID)
		{
			%foundID = true;
			
			%this.data[%a].delete();
			%this.data[%a] = "";
			
			continue;
		}
		
		if(%foundID)
		{
			%this.data[%a - 1] = %this.data[%a];
			%this.data[%a] = "";
		}
	}
	
	%this.dataCount--;
	
	return true;
}

function Sassy::getData(%this, %ID)
{
	for(%a = 0; %a <= %this.dataCount; %a++)
		if(%this.data[%a].ID == %ID)
			return %this.data[%a];
	
	return -1;
}

function SassyData::onAdd(%this)
{
	%errorStack = "";
	
	if(%this.ID $= "")
		%errorStack = "ERROR: SassyData::onAdd(): ID variable not specified!";
	
	if(!isObject(%this.parent))
		%errorStack = %errorStack @ ((%errorStack !$= "") ? "\n" : "") @ "ERROR: SassyData::onAdd(): Parent object not found!";
	
	if(%errorStack !$= "")
	{
		echo(%errorStack);
		
		return false;
	}
	
	for(%a = 1; %a <= %this.parent.valueCount; %a++)
		%this.value[%this.parent.value[%a]] = %this.parent.defaultValue[%a];
	
	return true;
}

function Sassy_loadPrefArray(%dataFile)
{
	if(isFile(%dataFile) == 0)
	{
		echo("ERROR: Sassy_loadPrefArray::loadData(): File '" @ %dataFile @ "' not found.");
		
		return false;
	}
	
	%file = new fileObject();
	%file.openForRead(%dataFile);
	
	%currentState = "";
	
	%prefCount = 0;
	%defaultPref = "";
	%ID = "";
	
	while(%file.isEOF() == 0)
	{
		%line = %file.readLine();
		
		if(getWord(%line, 0) $= "" && getWord(%line, 1) $= "")
			continue;
		
		if(getSubStr(%line, 0, 1) $= " ")
		{
			%line = getSubStr(%line, 1, strLen(%line) - 1);
			
			if(%currentState $= "prefs")
			{
				%prefCount++;
				
				%prefTag[getWord(%line, 0)] = %prefCount;
				%pref[%prefCount] = getWord(%line, 1);
				%defaultPref[%prefCount] = getWords(%line, 2, getWordCount(%line) - 1);
			}
			
			if(%currentState $= "ID")
			{
				%pref = %pref[%prefTag[getWord(%line, 0)]];
				
				%value = getWords(%line, 1, getWordCount(%line) - 1);
				
				if(%value $= "true" || %value $= "false")
					eval(strReplace(%pref, "%ID", %ID) @ " = " @ %value @ ";");
				else
					eval(strReplace(%pref, "%ID", %ID) @ " = \"" @ %value @ "\";");
			}
			
			continue;
		}
		
		%currentState = getWord(%line, 0);
		
		if(getWord(%line, 0) $= "ID")
		{
			%ID = getWord(%line, 1);
			
			for(%a = 1; %a <= %prefCount; %a++)
			{
				%value = %defaultPref[%a];
				
				if(%value $= "true" || %value $= "false")
					eval(strReplace(%pref, "%ID", %ID) @ " = " @ %value @ ";");
				else
					eval(strReplace(%pref, "%ID", %ID) @ " = \"" @ %value @ "\";");
			}
		}
	}
	
	%file.close();
	%file.delete();
	
	return true;
}